function [post_prob_h1]=compare_two_algorithms(dsets_results,n_folds)
%%function [post_prob_h1]=compare_two_algorithms(dsets_results,n_folds)
%
%Runs inference based on the Bayesian t-test and the Poisson-binomial for
%comparing two cross-validated algorithms on multiple data sets.
%
%--Input
%
%--dset_results is a matrix containing the difference of performance between
%two competing algorithms on each fold of each data set.
%Assume 10 runs of 10-folds cross-validation are performed on each data set.
%Then dset_results has lenght (100*m), where m is the number of data sets.
%The matrix has two columns:
%a)the dset id
%b)the difference between the results of the first algorithm and of the
%second algorithm
%
%--n_folds is the number of folds adopted for cross-validation (assumed equal
%for all data sets), from which the correlation of the cross-validation samples is computed
%as 1/n_folds.
%
%--Output
%The posterior probability of the mean difference of score between the two algorithms to be larger than
%0 on the considered collection of data sets, accounting for the
%uncertainty and the correlation  the cross-validation results on each data set.

%inits
rho=1/n_folds;

%helping vars
dset_id=dsets_results(:,1);
dsets_list=unique(dset_id);

if length(dsets_results)<length(dsets_list)*n_folds
    error (strcat('Number of results inferior to the product of the number of distinct data sets and n_folds.',...
        'The dset_id should be the first column of dsets_results.'));
end


for j=1:length(dsets_list)
    current_dset=dsets_list(j);
    idx= dsets_results(:,1)==current_dset;
    prob_h1_each_dset(j)=ttest_Bayesian(dsets_results(idx,2),rho,0);
end

[pdf,cdf]=poisson_bin(prob_h1_each_dset);
post_prob_h1=1-cdf(floor(length(prob_h1_each_dset)/2)+1);

end
