function [ p ] = ttest_Bayesian(x,rho,mu0)
%%[p] = ttest_Bayesian(x,rho,mu0)
%p is the posterior probability that the mean is greater than mu0.
%Usually you set mu0=0.
%
%rho is the correlation. For k-folds cross-validation, rho=n_te/n where
%n_te is the number of instances in the test set and n the total number of
%instances in the data set. For k=10, rho=1/10.
%
%The test uses a non-informative prior under which it matches the frequentist
%correlated t-test (Nadeau and Bengio, Mach Learning 2003).
%
%Given the correlation rho and the correlated observations x, it returns
%the posterior probability of the mean being larger than mu0.
%
%Example:
%ttest_Bayesian(x,1/10,0) return the probability of the mean being larger
%than 0, given correlation 1/10.
%


%parameters of the non-informative prior, to match the results of the
%frequentist correlated t-test
k0=1000000;
a=-0.5;
b=0;

m=0;

n=length(x);
H=ones(n,1);

M=rho*ones(n,n)+diag(ones(n,1))*(1-rho);

tildemu=inv((H'*inv(M)*H+1/k0))*(H'*inv(M)*x+mu0/k0);
hatmu=inv((H'*inv(M)*H))*(H'*inv(M)*x);
tildea=a+n/2;
tildeb0=0.5*((x-H*hatmu)'*inv(M)*(x-H*hatmu)+2*b-hatmu^2*H'*inv(M)*H-mu0^2/k0+tildemu^2*(H'*inv(M)*H+1/k0));

tildeb=max(tildeb0,0); %b+(x'*inv(M)*x-(H'*inv(M)*x+mu0/k0)'*inv(H'*inv(M)*H+1/k0)*(H'*inv(M)*x+mu0/k0))/2;

sigma_student=sqrt((tildeb/tildea/(H'*inv(M)*H+1/k0))); %(n/(1+(n-1)*rho)+1/k0)

t=(tildemu-m)/sigma_student;


%the posterior of the t statistics is integrated
%from -inf to the observed value, in order to the the prob(mean>0).
p=tcdf(t,2*tildea); %probability that the mean is greater than zero

if isnan(p)
    p=0.5;
end

end

