function [cv_results]=cv(nruns,nfolds,n,theta)
%%[results]=cv(nruns,nfolds,n,theta)
%implements cross-validation, given the number or runs, folds, 
%data set size (n) and theta. Compares nbc and zeror classifiers.
%Theta determines the conditional prob of the feature while
%generating the data. As a consequence the expected accuracy of naive Bayes is
%theta. If theta=0.5, an irrelevant feature is generated. 
%Both classifiers are random guessers and detected significance are type I
%errors.
%cv_results contains:
%-nbc accuracy
%-zeror accuracy
%-delta = nbc accuracy - zeror accuracy
%-var= variance  (nbc accuracy - zeror accuracy)

if nargin<1
    nruns=10;
end
if nargin<2
    nfolds=5;
end
if nargin<3
    n=1000; %total sample size
end
if nargin<4
    theta=0.5;
end

data=generate_data(n,theta);


for i=1:nruns
    index=random_index(data);
    if (i==1)
        cv_results=c_val(index,nfolds);
    else
        tmp=c_val(index,nfolds);
        cv_results.nbc=[cv_results.nbc; tmp.nbc];
        cv_results.zeror=[cv_results.zeror; tmp.zeror];
        cv_results.delta=[cv_results.delta; tmp.delta];
        cv_results.var=[cv_results.var; tmp.var];
    end
end

    function [cv]=c_val(index,nfolds)
        cv=struct;
        cv.nbc=ones(nfolds,1);
        cv.zeror=cv.nbc;
        cv.delta=cv.nbc;
        cv.var=cv.nbc;
        for j=1:nfolds
            train=data(index~=j,:);
            test=data(index==j,:);
            nbc=learn_nbc(train);
            zeror=learn_zeror(train);
            preds_nbc=test_nbc(nbc,test);
            preds_zeror=test_zeror(zeror,test);
            %acc_nbc
            nbc_correct=preds_nbc==test(:,1);
            cv.nbc(j)=mean(nbc_correct);
            %acc_zero
            zeror_correct=preds_zeror==test(:,1);
            cv.zeror(j)=mean(zeror_correct);
            %debug
            %disp([tmp_res(j,1) tmp_res(j,2)]);
            %delta_acc
            cv.delta(j)=mean(nbc_correct-zeror_correct);
            %var_delta_acc
            cv.var(j)=var(nbc_correct-zeror_correct);
        end
    end


    function zeror=learn_zeror(train)
        zeror.prediction=1;
        n1=sum((train(:,1))==1);
        n2=sum((train(:,1))==2);
        if n2>n1
            zeror.prediction=2;
        end
    end
    

    function preds_zeror=test_zeror(zeror,test)
        preds_zeror=zeros(length(test),1);
        preds_zeror=preds_zeror + zeror.prediction;
    end

   

    function idx=random_index(data_in)
        p=randperm(length(data_in));
        data_in=data_in(p,:);
        %sort data according to class
        data_in=sortrows(data_in,2);
        
        %idx says to which fold each instance belong.
        
        %assign each instance to a fold.
        tmp_idx=1:nfolds;
        idx=tmp_idx;
        while (length(idx)<length(data_in))
            idx=cat(2,idx,tmp_idx);
        end
        idx=idx(1:length(data_in));
    end
end