function [results]=multiple_dsets_fixed_delta(delta_acc,how_many_dsets,n_runs,n_folds,reps)
%[results]=multiple_dsets_fixed_delta(delta_acc,how_many_dsets,nruns,n_folds,reps)
%
%Compares the Poisson and Wilcoxon-signed rank test doing inference on multiple data sets.
%Saves the results to csv and mat file, besided on the returned struct
%named results.
%
%-delta_acc is the actual diff of probability between two classifiers,
%assumed equal on all data sets
%
%-how_many_dsets: how many dsets are used for comparing A and B over
%multiple data sets
%
%-n_runs, n_folds denote the number of runs and folds of cross-validation
%
%-reps:how many repetitions of the experiments.


sample_sizes=[25 50 100 250 500 1000];
%set the seed for reproducible results
randn('seed',0);


if delta_acc>=0.5
    error('delta_acc should be <= 0.5');
end

if delta_acc<0
    error('delta_acc should be > 0');
end

counter=1;
for j=1:length(delta_acc)
    for k=1:length(how_many_dsets)
        settings(counter).delta_acc=delta_acc(j);
        settings(counter).dsets=how_many_dsets(k);
        counter=counter+1;
    end
end

%helping variables
rho=1/n_folds;
%inits
ttest_poisson_h1=ones(reps,1);
sign_rank_p_value=ttest_poisson_h1;
results(length(settings)).sample_size=[];



%simulation
for j=1:length(settings)
    disp('setting');
    disp(j);
    current_many_dsets = settings(j).dsets;
    current_delta_acc=settings(j).delta_acc;
    diff_acc_a_b_each_dset=ones(current_many_dsets,1);
    dset_sizes=ones(length(how_many_dsets),1);
    
    ttest_prob_h1=ones(current_many_dsets,1);
    
    for k=1:reps
        if mod(k,100)==0;
            disp('reps');
            disp(k);
        end
        
        Diff_ab=zeros(n_runs*n_folds,current_many_dsets);
        for i=1:current_many_dsets
            cv_results.delta=0;
            idx=randperm(length(sample_sizes));
            current_size=sample_sizes(idx(1));
            dset_sizes(i)=current_size;
            while var(cv_results.delta)==0 && mean(cv_results.delta)==0
                %notice 0.5+diff_vector(i). The accuracy of zeror is 0.5,
                %the expected accuracy of NBC is 0.5+diff_vector(i), so the
                %expected difference of accuracy is diff_vector(i)
                [cv_results]=cv(n_runs,n_folds,current_size,0.5+delta_acc);
            end
            
            diff_a_b=cv_results.delta;
                                    
            diff_acc_a_b_each_dset(i)=mean(diff_a_b);
            
            %equivalent to p-value one-sided right tailed
            ttest_prob_h1(i)=ttest_Bayesian(diff_a_b,rho,0);
            Diff_ab(:,i)=diff_a_b;
            
            
        end
     
        [pdf,cdf]=poisson_bin(ttest_prob_h1);
        
        %the first element of pdf is
        %probability of 0 successes; the last element is the probability of
        %success in every data set. We integrate the pdf of the poission-bin
        %over the following domain: the number of dsets > n/2, obtaining
        %the overall probability of the second algorithm being better than
        %the first.
        ttest_poisson_h1(k)=1-cdf(floor(current_many_dsets/2)+1);       
        
        %frequentist Wilcoxon signed rank
        sign_rank_p_value(k) = signrank(diff_acc_a_b_each_dset,0,'tail','right');
        [~,student_p_value(k)]=ttest(diff_acc_a_b_each_dset,0,'tail','right');
    end
    
    results(j).how_many_dsets=current_many_dsets;
    results(j).delta_acc=current_delta_acc;
    results(j).ttest_poisson_h1=ttest_poisson_h1;
    results(j).sign_rank_p_value=sign_rank_p_value;
   
    
end
%save the matlab workspace
tmp_str=strcat(num2str(n_runs),'runs',num2str(n_folds),'folds_','theta',num2str(delta_acc(1)),...
    '_dsets',num2str(how_many_dsets(1)));
filename=strcat('multiple_dsets_fixed_delta_',tmp_str,'.csv');


%save to txt file
save_results(results,filename);
mat_filename=strcat('multiple_dsets_delta_acc_',tmp_str,'.mat');


save (mat_filename, 'results');

    function save_results(results,filename)
        fid = fopen(filename,'w');
        fprintf(fid,'%s  \n','delta_acc,cv_runs,cv_folds,num_experiments,how_many_dsets,h1_poisson,h1_sign_rank');
        for ii=1:length(results)
            num_experiments=length(results(ii).ttest_poisson_h1);
            h1_poisson=mean(results(ii).ttest_poisson_h1>.95);
            h1_sign_rank=mean(results(ii).sign_rank_p_value<.05);
            tmp_vector=[h1_poisson;h1_sign_rank];
            
            fprintf(fid,'%1.2f %s %d %s %d %s %d %s %d %s %1.2f %s',results(ii).delta_acc,',',n_runs,',',n_folds,',',num_experiments,',',results(ii).how_many_dsets,',');
            for jj=1:length(tmp_vector)-1
                fprintf(fid, '%1.3f %s', tmp_vector(jj),',');
            end
            fprintf(fid, '%1.3f', tmp_vector(end));
            fprintf(fid, '%s', ',');
        end
        fclose(fid);
    end

end
