function [decision_poisson,decision_sign_rank,p_value_sign_rank,post_prob_h1_poisson]=analyze_uci(class_id,dset_id,score,exp_type)
%function [p_value_sign_rank,decision_sign_rank,post_prob_h1_poisson,decision_poisson]=analyze_uci(class_id,dset_id,score,exp_type)
%reports the results of Poisson and signed-rank test on UCI dsets
%performing all pairwise comparisons
%exp_type can be 'first_half', 'second_half' or 'full'
%
%Produces two csv files, containing the decision of each test for each couple of classifiers.
%Such files contain also the classifier names.
%The returned matlab matrix are symmetric. The redudant entries of the
%matrix are filled with NaN.
%
%--Example
%[decision_poisson,decision_sign_rank,p_value_sign_rank,post_prob_h1_poisson]=analyze_uci(ClassifiedID,DatasetID,Percent_correct,'full')
%The returned arguments contains the matrixes of the decision, p-values and
%probabilities. Additionally csv files are produced.
%

%skip first header row
% class_id=class_id(2:end);
% dset_id=dset_id(2:end);
% score=score(2:end);

%inits
n_folds=10;%hard-coded
rho=1/n_folds;
classifier_names={'naive Bayes';'aode';'hnb';'j48';'j48_grafted'};
p_value_sign_rank=nan*ones(length(classifier_names),length(classifier_names));
post_prob_h1_poisson=p_value_sign_rank;
decision_sign_rank=p_value_sign_rank;
decision_poisson=p_value_sign_rank;

%helping vars
dsets_list=unique(dset_id);
if strcmp(exp_type,'first_half')
    dsets_list=dsets_list(1:round(length(dsets_list)/2));
elseif  strcmp(exp_type,'second_half')
    dsets_list=dsets_list(round(length(dsets_list)/2)+1:end);
elseif ~strcmp(exp_type,'full')
    error('wrong parameter exp_type');
end
  
sign_rank_reversed_sign=[];

%implement the pairwise comparison
for i=1:length(classifier_names)-1
    for j=i+1:length(classifier_names)
        %sign-rank
        [avg_score_i,avg_score_j]=data_sign_rank(i,j);
        p_value_sign_rank(i,j)=signrank(avg_score_i,avg_score_j,'tail','right');
        if mean(avg_score_j)>mean(avg_score_i)
            p_value_sign_rank(i,j)=signrank(avg_score_j,avg_score_i,'tail','right');
            sign_rank_reversed_sign=[sign_rank_reversed_sign;strcat(num2str(i),num2str(j))];
        end
        %poisson
        post_prob_h1_poisson(i,j)=poisson(i,j);
    end
end

decision_sign_rank(~isnan(p_value_sign_rank))=p_value_sign_rank(~isnan(p_value_sign_rank))<0.05;
decision_poisson(~isnan(post_prob_h1_poisson))=post_prob_h1_poisson(~isnan(post_prob_h1_poisson))>0.95;

filename_poisson=strcat('decisions_poisson_',exp_type,'.txt');
filename_sign_rank=strcat('decisions_sign_rank_',exp_type,'.txt');
matrix2latex(decision_poisson,filename_poisson,'rowLabels',classifier_names,'columnLabels',classifier_names);
matrix2latex(decision_sign_rank,filename_sign_rank,'rowLabels',classifier_names,'columnLabels',classifier_names);

    function [avg_score_i,avg_score_j]=data_sign_rank(class_i,class_j)
        avg_score_i=ones(length(dsets_list),1);
        avg_score_j=ones(length(dsets_list),1);
        for ii=1:length(dsets_list)
            avg_score_i(ii)=mean(score(class_id==i & dset_id==ii));
            avg_score_j(ii)=mean(score(class_id==j & dset_id==ii));
        end
    end

    function post_prob_h1= poisson (class_i,class_j)
        results=[class_id dset_id score];
        results_class_i=results(results(:,1)==class_i,:);
        results_class_j=results(results(:,1)==class_j,:);
        prob_h1_each_dset=ones(length(dsets_list),1);
        
        
        for jj=1:length(dsets_list)
            tmp_results_class_i=results_class_i(results_class_i(:,2)==jj,3);
            tmp_results_class_j=results_class_j(results_class_j(:,2)==jj,3);
            prob_h1_each_dset(jj)=ttest_Bayesian(tmp_results_class_j-tmp_results_class_i,rho,0);
            if mean(avg_score_i)>mean(avg_score_j)
                prob_h1_each_dset(jj)=ttest_Bayesian(tmp_results_class_i-tmp_results_class_j,rho,0);
            end
        end
        [pdf,cdf]=poisson_bin(prob_h1_each_dset);
        post_prob_h1=1-cdf(floor(length(prob_h1_each_dset)/2)+1);
    end
end